const attributes = {
    fly: { id: "palladium:levitation_speed", uuid: "8214453b-56ff-42d6-bb10-2af705a5b53c", minLevel: 6, maxLevel: 9, cap: 1.0 },
    armor: { id: "minecraft:generic.armor", uuid: "8e8d9ddb-a885-4f11-8bf8-4935f5b9b224", minLevel: 4, maxLevel: 9, cap: 20 },
    damage: { id: "minecraft:generic.attack_damage", uuid: "71a9d77e-8f23-4888-a634-fd05b6cc0761", minLevel: 4, maxLevel: 9, cap: 4 },
    speed: { id: "minecraft:generic.movement_speed", uuid: "296d32fa-6bd3-40f4-a70f-d45834541e2f", minLevel: 5, maxLevel: 9, cap: 0.05 },
};

StartupEvents.registry('palladium:abilities', (event) => {
    event.create('telekinesis/boost_ability')

        .firstTick((player, entry, holder, enabled) => {
            if (enabled) {
                player.persistentData.telekinesisEntityHidenBy = "";
                if (global.telekinesisConfig.generalGlowingEffect) {
                    let playerTeam = player.teamId
                    if (playerTeam && playerTeam != "telekinesis") {
                        player.persistentData.telekinesisOldTeam = playerTeam;
                    }
                    player.server.runCommandSilent(`team join telekinesis ${player.username}`);
                    player.setGlowing(true);
                }
                global.playSoundLocal(player, 'minecraft:block.enchantment_table.use', "players", 0.5, 1.5);

            } else {
                player.tags.remove("telekinesis_boost_toggle")
                player.stages.remove("telekinesis.boost.toggle")
                resetAttributesAndTeam(player);
            }
        })

        .tick((player, entry, holder, enabled) => {
            if (enabled) {
                const xp = "telekinesis.psy_xp";
                const energy = "telekinesis.psy_energy";
                let psycoLevel = palladium.scoreboard.getScore(player, "telekinesis.psy_level", 1);
                let sharing1 = player.tags.contains("telekinesis_boost_sharing_slot4")
                let sharing2 = player.tags.contains("telekinesis_boost_sharing_slot5")
                let sharingValue = (sharing1 ? 1 : 0) + (sharing2 ? 1 : 0) + 1;
                applyAttributes(player, psycoLevel, sharingValue);
                
                palladium.scoreboard.subtractScore(player, energy, 1);
                if (palladium.scoreboard.getScore(player, energy) <= 0) {
                    palladium.scoreboard.setScore(player, energy, -1);
                }
                
                palladium.scoreboard.addScore(player, xp, 3);
                player.persistentData.telekinesisBoostSharingFrom = ""
                player.persistentData.telekinesisEnergyCooldown = 40;

                
                player.resetFallDistance();
            }
        })

        .lastTick((player, entry, holder, enabled) => {
            player.tags.remove("telekinesis_boost_toggle")
            player.stages.remove("telekinesis.boost.toggle")
            resetAttributesAndTeam(player);
        });
});


function applyAttributes(player, level, divide) {
    for (const attr of Object.values(attributes)) {
        player.removeAttribute(attr.id, attr.uuid);
    }
    for (const attr of Object.values(attributes)) {
        if (level >= attr.minLevel) {
            
            let progression = Math.min(
                (level - attr.minLevel) / (attr.maxLevel - attr.minLevel),
                1
            ); 
            let value = (progression * attr.cap) / divide;

            
            player.modifyAttribute(attr.id, attr.uuid, value, "addition");
        }
    }
}


function resetAttributesAndTeam(target) {
    
    for (const attr of Object.values(attributes)) {
        target.removeAttribute(attr.id, attr.uuid);
    }
    
    if (!target.tags.contains("telekinesis_boost_toggle")) {
        target.setGlowing(false);
        if (global.telekinesisConfig.generalGlowingEffect) {
            
            Utils.server.scheduleInTicks(2, () => {
                let targetId = target.isPlayer() ? target.username : target.uuid;
                target.server.runCommandSilent(`team leave ${targetId}`);
            });
        }
    }

}