const $Projectile = Java.loadClass('net.minecraft.world.entity.projectile.Projectile');

StartupEvents.registry('palladium:abilities', (event) => {
    event.create('telekinesis/shield_ability')

        .firstTick((player, entry, holder, enabled) => {
            if (enabled) {
                player.persistentData.telekinesisRepulse = true
                global.playSoundLocal(player, 'minecraft:block.enchantment_table.use', "players", 0.5, 1.5);
            }
        })

        .tick((player, entry, holder, enabled) => {
            if (!enabled) return;

            const energy = "telekinesis.psy_energy";
            const xp = "telekinesis.psy_xp";
            const level = "telekinesis.psy_level";

            
            palladium.scoreboard.subtractScore(player, energy, 2);
            if (palladium.scoreboard.getScore(player, energy) <= 0) {
                palladium.scoreboard.setScore(player, energy, -1);
            }

            
            palladium.scoreboard.addScore(player, xp, 2);

            let playerLevel = palladium.scoreboard.getScore(player, level);
            let playerAABB = player.getBoundingBox().inflate(5);

            
            player.level.getEntitiesWithin(playerAABB).forEach(entity => {
                if (shouldAffectEntity(player, entity)) {
                    handleProjectile(player, entity, energy);
                }
                if (playerLevel >= 5 && shouldAffectLivingEntity(player, entity)) {
                    handleLivingEntity(player, entity);
                }
            });

            player.persistentData.telekinesisEnergyCooldown = 40;
        })
});


function shouldAffectEntity(player, entity) {
    return (
        entity instanceof $Projectile &&
        !entity.persistentData?.psyco_shield &&
        (entity.nbt.inGround == null || entity.nbt.inGround == 0) &&
        !entity.tags.contains("psyco_shield_drop") &&
        (entity.persistentData?.telekinesisTargetedBy != player.uuid) &&
        isEntityInFov(player, entity, 45, 22.5)
    );
}


function handleProjectile(player, entity, energy) {
    const distance = calculateDistance(player, entity);
    const speed = calculateSpeed(entity);

    if (speed > 2) {
        normalizeMotion(entity, speed);
    }

    if (distance <= 2.5) {
        immobilizeProjectile(entity);
        entity.playSound('minecraft:item.crossbow.hit', 0.5, 2)
        palladium.scoreboard.subtractScore(player, energy, 15);
        if (palladium.scoreboard.getScore(player, energy) <= 0) {
            palladium.scoreboard.setScore(player, energy, -1);
        }
    }
}


function immobilizeProjectile(entity) {
    entity.mergeNbt({ NoAI: 1, life: 0 });
    entity.setMotion(entity.motionX / 500, entity.motionY / 500, entity.motionZ / 500);
    entity.setNoGravity(true);
    if(global.telekinesisConfig.generalGlowingEffect) entity.setGlowing(true);
    entity.persistentData.psyco_shield = 40;
}


function normalizeMotion(entity, speed) {
    entity.setMotion(
        (entity.motionX / speed) * 2,
        (entity.motionY / speed) * 2,
        (entity.motionZ / speed) * 2
    );
}


function calculateDistance(player, entity) {
    const dx = entity.x - player.x;
    const dy = entity.y - player.eyeY;
    const dz = entity.z - player.z;
    return Math.sqrt(((dx) ** 2) + ((dy) ** 2) + ((dz) ** 2));
}


function calculateSpeed(entity) {
    return Math.sqrt(((entity.motionX) ** 2) + ((entity.motionY) ** 2) + ((entity.motionZ) ** 2));
}

function isEntityInFov(player, entity, yawFovThreshold, pitchFovThreshold) {
    
    let dx = entity.x - player.x;
    let dy = entity.y - player.eyeY;
    let dz = entity.z - player.z;
    
    let totalDistance = calculateDistance(player, entity);

    
    let minDistance = 1; 
    if (totalDistance < minDistance) {
        return false; 
    }

    
    let playerYawRad = player.getYaw() * (JavaMath.PI / 180); 
    let playerPitchRad = player.getPitch() * (JavaMath.PI / 180); 

    let playerDirection = [
        -Math.sin(playerYawRad) * Math.cos(playerPitchRad), 
        -Math.sin(playerPitchRad),                         
        Math.cos(playerYawRad) * Math.cos(playerPitchRad), 
    ];

    
    let entityDirection = [
        dx / totalDistance, 
        dy / totalDistance, 
        dz / totalDistance, 
    ];

    
    let dotProduct =
        playerDirection[0] * entityDirection[0] +
        playerDirection[1] * entityDirection[1] +
        playerDirection[2] * entityDirection[2];

    
    if (dotProduct <= 0.1) {
        return false; 
    }

    
    let angleBetween = Math.acos(dotProduct) * (180 / JavaMath.PI); 

    
    return angleBetween <= Math.max(yawFovThreshold, pitchFovThreshold);
}


function shouldAffectLivingEntity(player, entity) {
    return (
        entity.isLiving() &&
        !entity.entityType.is(banned_entities) &&
        entity != player &&
        (entity.persistentData?.telekinesisTargetedBy != player.uuid) &&
        isEntityInFov(player, entity, 45, 75)
    );
}


function handleLivingEntity(player, entity) {
    const distance = calculateDistance(player, entity);

    if (distance > 4) {
        player.persistentData.telekinesisRepulse = true
    }
    if (distance <= 4) {

        let dx = entity.x - player.x;
        let dy = entity.y - player.y;
        let dz = entity.z - player.z;

        let magnitude = Math.sqrt(dx * dx + dy * dy + dz * dz);
        const force = 0.5; 

        
        entity.setMotion(
            (dx / magnitude) * force,
            0.1, 
            (dz / magnitude) * force
        );

        if (entity.isPlayer()) {
            entity.connection.send(new $ClientboundSetEntityMotionPacket(entity));
        }
        if(player.persistentData.telekinesisRepulse == true){
            entity.playSound('minecraft:ui.toast.in', 2, 2)
            player.persistentData.telekinesisRepulse = false;
        }
    }
}