global.telekinesisConfig = {};


const CONFIG_PATH = 'config/telekinesis.json';

const DEFAULT_CONFIG = {
    _comments: [
        "enableCauldronFormula: Enable or disable the cauldron functionality.",
        "formulaItems: List of items required for the cauldron formula.",
        "firstLogAcquisitionChance: Percentage chance for acquiring the power on first login: 0 to 100.",
        "generalGlowingEffect: Enables the glowing effect on entities affected by telekinesis.",
        "hidenEntityGlowingEffect: Enables the glowing effect on entities hidden by a block during telekinesis.",
        "powerToReplace: Defines the power that will be replaced: packID:powerID"
    ],
    enableCauldronFormula: true,
    formulaItems: [
        "minecraft:nether_star",
        "minecraft:glowstone_dust",
        "minecraft:amethyst_shard"
    ],
    firstLogAcquisitionChance: 0,
    generalGlowingEffect: true,
    hidenEntityGlowingEffect: true,
    powerToReplace: ""
};


StartupEvents.init((event) => {
    console.log("[Config] Loading telekinesis configuration...");

    try {
        
        let config = JsonIO.read(CONFIG_PATH);

        if (!config) {
            throw new Error('Configuration file is empty or missing.');
        }

        
        let fixedConfig = validateAndFixConfig(config);

        
        if (!areConfigsEqual(config, fixedConfig)) {
            console.warn("[Config] Configuration file had invalid or missing keys. Updating configuration...");
            JsonIO.write(CONFIG_PATH, fixedConfig);
        }

        global.telekinesisConfig = fixedConfig; 
    } catch (error) {
        console.warn(`[Config] Configuration file not found or invalid. Creating default config at: ${CONFIG_PATH}`);
        global.telekinesisConfig = DEFAULT_CONFIG; 
        JsonIO.write(CONFIG_PATH, DEFAULT_CONFIG); 
    }

    console.log(`[Config] Configuration Loaded.`);
});


function validateAndFixConfig(config) {
    let fixedConfig = {}; 

    for (let key in DEFAULT_CONFIG) {
        
        if (config.hasOwnProperty(key)) {
            fixedConfig[key] = config[key];
        } else {
            fixedConfig[key] = DEFAULT_CONFIG[key];
        }
    }

    return fixedConfig;
}


function areConfigsEqual(config1, config2) {
    
    for (let key in DEFAULT_CONFIG) {
        if (JSON.stringify(config1[key]) !== JSON.stringify(config2[key])) {
            return false; 
        }
    }
    return true; 
}


function updateConfig(newConfig) {
    let validatedConfig = validateAndFixConfig(newConfig); 
    JsonIO.write(CONFIG_PATH, validatedConfig); 
    global.telekinesisConfig = validatedConfig; 
    console.log(`[Config] Configuration updated.`);
}
