const abilities = ["psyco_ability", "boost_ability", "shield_ability", "slot4_sharing", "slot5_sharing"];
const levelData = {
    1: { energyMax: 200, xpMax: 4000 },
    2: { energyMax: 400, xpMax: 8000 },
    3: { energyMax: 600, xpMax: 16000 },
    4: { energyMax: 800, xpMax: 32000 },
    5: { energyMax: 1000, xpMax: 64000 },
    6: { energyMax: 1200, xpMax: 128000 },
    7: { energyMax: 1400, xpMax: 256000 },
    8: { energyMax: 1600, xpMax: 512000 },
    9: { energyMax: 1800, xpMax: 1024000 }
};

PlayerEvents.loggedIn(event => {
    const player = event.player;

    
    if (!player.stages.has("first_log_completed")) {
        
        player.stages.add("first_log_completed");

        
        const randomChance = Math.random() * 100;

        
        if (randomChance <= global.telekinesisConfig.firstLogAcquisitionChance) {
            superpowerUtil.addSuperpower(player, "telekinesis:telekinesis");
            player.tell(Text.of("What is this strange power?").color(0xe300cc).bold(true));

            player.potionEffects.add("nausea", 215, 2, true, false);
            player.potionEffects.add("slowness", 150, 2, true, false);
            player.potionEffects.add("blindness", 150, 2, true, false);
            global.playSoundLocal(player, 'minecraft:entity.elder_guardian.curse', "players", 0.2, 0.1);
            player.persistentData.telekinesisDisease = 150;
        }
    }
});

PlayerEvents.tick(event => {
    const player = event.player;

    if (!!global.telekinesisConfig.powerToReplace) {
        if (superpowerUtil.hasSuperpower(player, global.telekinesisConfig.powerToReplace)) {
            superpowerUtil.removeSuperpower(player, global.telekinesisConfig.powerToReplace)
            superpowerUtil.addSuperpower(player, 'telekinesis:telekinesis')
        }
    }

    if (!superpowerUtil.hasSuperpower(player, 'telekinesis:telekinesis')) return;

    
    const persistentData = player.persistentData;
    let powerInUse = !!persistentData.telekinesisRequiredLevel;
    let psyDisease = !!persistentData.telekinesisDisease;
    let energyOnCooldown = !!persistentData.telekinesisEnergyCooldown;

    const scoreboard = player.getScoreboard();
    const objectives = {
        psyEnergyMax: scoreboard.getObjective('telekinesis.psy_energy_max'),
        psyEnergy: scoreboard.getObjective('telekinesis.psy_energy'),
        psyLevel: scoreboard.getObjective('telekinesis.psy_level'),
        psyXp: scoreboard.getObjective('telekinesis.psy_xp'),
        psyXpMax: scoreboard.getObjective('telekinesis.psy_xp_max')
    };
    let scores = {
        psyEnergyMax: scoreboard.getOrCreatePlayerScore(player.getUsername(), objectives.psyEnergyMax),
        psyEnergy: scoreboard.getOrCreatePlayerScore(player.getUsername(), objectives.psyEnergy),
        psyLevel: scoreboard.getOrCreatePlayerScore(player.getUsername(), objectives.psyLevel),
        psyXp: scoreboard.getOrCreatePlayerScore(player.getUsername(), objectives.psyXp),
        psyXpMax: scoreboard.getOrCreatePlayerScore(player.getUsername(), objectives.psyXpMax)
    };

    if (abilities.some(ability => abilityUtil.isEnabled(player, "telekinesis:telekinesis", ability))) {
        persistentData.telekinesisExhaustion = (persistentData.telekinesisExhaustion || 0) + 1;

        if (persistentData.telekinesisExhaustion >= 300) {
            
            player.setSaturation(Math.max(player.saturation - 1, 0));

            player.addExhaustion(1);
            persistentData.remove("telekinesisExhaustion");
        }
    }
    
    let currentLevel = scores.psyLevel.getScore();
    if (currentLevel > 0 && currentLevel <= 9) {
        let levelConfig = levelData[currentLevel];
        if (
            scores.psyEnergyMax.getScore() !== levelConfig.energyMax ||
            scores.psyXpMax.getScore() !== levelConfig.xpMax
        ) {
            scores.psyEnergyMax.setScore(levelConfig.energyMax);
            scores.psyXpMax.setScore(levelConfig.xpMax);

            
            if (scores.psyEnergy.getScore() > levelConfig.energyMax) {
                scores.psyEnergy.setScore(levelConfig.energyMax);
            }
        }
    } else if (currentLevel == 0) {
        
        scores.psyEnergyMax.setScore(levelData[1].energyMax);
        scores.psyEnergy.setScore(1);
        scores.psyLevel.setScore(1);
        scores.psyXp.setScore(0);
        scores.psyXpMax.setScore(levelData[1].xpMax);
    }

    
    if (
        scores.psyEnergy.getScore() < scores.psyEnergyMax.getScore() &&
        !powerInUse &&
        !psyDisease &&
        !energyOnCooldown &&
        player.foodLevel >= 6
    ) {
        let energyToAdd = Math.floor(scores.psyEnergyMax.getScore() * 0.01);
        scores.psyEnergy.add(energyToAdd);
    }

    
    if (scores.psyEnergy.getScore() < 0 && !psyDisease) {
        scores.psyEnergy.setScore(0);
        player.potionEffects.add("nausea", 215, 2, true, false);
        player.potionEffects.add("slowness", 150, 2, true, false);
        player.potionEffects.add("blindness", 150, 2, true, false);
        global.playSoundLocal(player, 'minecraft:entity.elder_guardian.curse', "players", 0.2, 0.1);
        persistentData.telekinesisDisease = 150;
    }

    
    if (psyDisease) {
        persistentData.telekinesisDisease -= 1;
        if (persistentData.telekinesisDisease <= 0) {
            persistentData.telekinesisDisease = ""; 
        }
    }

    
    if (energyOnCooldown) {
        persistentData.telekinesisEnergyCooldown -= 1;
        if (persistentData.telekinesisEnergyCooldown <= 0) {
            persistentData.telekinesisEnergyCooldown = ""; 
        }
    }

    
    if (scores.psyXp.getScore() >= scores.psyXpMax.getScore() && currentLevel < 9) {
        scores.psyLevel.setScore(currentLevel + 1);
        let newLevelConfig = levelData[currentLevel + 1];
        scores.psyXpMax.setScore(newLevelConfig.xpMax);
        scores.psyEnergyMax.setScore(newLevelConfig.energyMax);
        scores.psyXp.setScore(0);
        global.playSoundLocal(player, 'minecraft:block.note_block.chime', "players", 1, 0.1);
    }

    if(persistentData.telekinesisOldDim != player.level.dimension.toString()){
        if(scores.psyEnergy.getScore() > (scores.psyEnergy.getScore() - Math.floor(scores.psyEnergyMax.getScore() * 0.01))){
            scores.psyEnergy.setScore(scores.psyEnergy.getScore() - Math.floor(scores.psyEnergyMax.getScore() * 0.01));
        }
            refreshPlayerStages(player);
    }
    persistentData.telekinesisOldDim = player.level.dimension.toString();
});

function refreshPlayerStages(player) {
    
    let savedStages = [];
    
    
    player.stages.getAll().forEach(stage => {
        savedStages.push(stage);
    });

    
    Utils.server.scheduleInTicks(1, () => {
        
        savedStages.forEach(stage => player.stages.remove(stage));
        savedStages.forEach(stage => player.stages.add(stage));

    });
}
