const broke_falling = "telekinesis:broke_falling";
const $Projectile = Java.loadClass('net.minecraft.world.entity.projectile.Projectile');
const NETHER_STAR = "minecraft:nether_star";
const GLOWSTONE_DUST = "minecraft:glowstone_dust";
const AMETHYST_SHARD = "minecraft:amethyst_shard";


ServerEvents.tick((e) => {
    const entities = e.server.getEntities();
    entities.forEach(entity => processEntityTick(entity));
});


function processEntityTick(entity) {
    const entityData = entity.persistentData;

    if (global.telekinesisConfig.enableCauldronFormula && entity.type == "minecraft:item" && entity.item.id == global.telekinesisConfig.formulaItems[0]) {
        const inCauldron = entity.level.getBlock(entity.blockX, entity.blockY, entity.blockZ);
        
        if (isWaterCauldron(inCauldron)) {
            checkItemsInCauldron(entity, inCauldron);
        }
    }

    
    if (entityData?.telekinesisTargetedBy) {
        handleTelekinesisTarget(entity, entityData);
    } else if (entityData?.telekinesisEntityHidenBy) {
        handleHiddenEntity(entity, entityData);
    }

    
    if (entityData?.psyco_shield) {
        handlePsycoShield(entity, entityData);
    }

    
    if (isTelekinesisFallingBlock(entity)) {
        handlePsycoBlock(entity);
    }

    if (entityData?.telekinesisBoostSharingFrom) {
        let fromPlayer = getEntityByUUID(entityData.telekinesisBoostSharingFrom);
        if (!fromPlayer || distanceBetween(fromPlayer, entity) >= 40 || entity.level != fromPlayer.level) {
            resetAttributesAndTeam(entity)
            const entityId = entity.isPlayer() ? entity.username : entity.uuid;
            const oldTeam = entityData.telekinesisOldTeam;
            if (oldTeam && oldTeam !== "telekinesis") {
                Utils.server.runCommandSilent(`team join ${oldTeam} ${entityId}`);
            }
            entityData.telekinesisBoostSharingFrom = ""
        }
    }
    if (entityData?.telekinesisBoostSharingSlot4) {
        let toPlayer = getEntityByUUID(entityData.telekinesisBoostSharingSlot4);
        let nameSlot4 = entityData.telekinesisBoostSharingSlot4Name
        if ((!toPlayer && nameSlot4) || ((distanceBetween(toPlayer, entity) >= 40) && nameSlot4) || (entity.level != toPlayer.level && nameSlot4)) {
            let nameToStage = nameSlot4.toLowerCase().replace(/_/g, ".")
            let slot4 = "slot4" + nameToStage
            entity.stages.remove(slot4)
            entity.tags.remove("telekinesis_sharing_slot4")
            entityData.telekinesisBoostSharingSlot4Name = ""
            entityData.telekinesisBoostSharingSlot4 = ""
        }
    }
    if (entityData?.telekinesisBoostSharingSlot5) {
        let toPlayer = getEntityByUUID(entityData.telekinesisBoostSharingSlot5);
        let nameSlot5 = entityData.telekinesisBoostSharingSlot5Name
        if ((!toPlayer && nameSlot5) || ((distanceBetween(toPlayer, entity) >= 40) && nameSlot5) || (entity.level != toPlayer.level && nameSlot5)) {
            let nameToStage = nameSlot5.toLowerCase().replace(/_/g, ".")
            let slot5 = "slot5" + nameToStage
            entity.stages.remove(slot5)
            entity.tags.remove("telekinesis_sharing_slot5")
            entityData.telekinesisBoostSharingSlot5Name = ""
            entityData.telekinesisBoostSharingSlot5 = ""
        }
    }
}


function isTelekinesisFallingBlock(entity) {
    return entity.getType().includes('falling_block') &&
        entity.tags.contains("psyco_block");
}


function handleTelekinesisTarget(entity, entityData) {
    const player = getEntityByUUID(entityData.telekinesisTargetedBy);
    if (!player || (player.persistentData?.telekinesisTarget != entity.uuid &&
        player.persistentData?.telekinesisBlockTarget != entity.uuid)) {
        handleTargetedEntity(entity);
    }
}


function handleHiddenEntity(entity, entityData) {
    const player = getEntityByUUID(entityData.telekinesisEntityHidenBy);
    if (!player || player.persistentData?.telekinesisEntityHiden != entity.uuid) {
        entity.setGlowing(false);
        entityData.remove("telekinesisEntityHidenBy");
    }
}


function handleTargetedEntity(entity) {
    if (entity.type == "minecraft:armor_stand") {
        entity.kill();
    } else {
        entity.setNoGravity(false);
        entity.setGlowing(false);

        
        Utils.server.scheduleInTicks(1, () => {
            const entityId = entity.isPlayer() ? entity.username : entity.uuid;
            Utils.server.runCommandSilent(`team leave ${entityId}`);
            const oldTeam = entity.persistentData?.telekinesisOldTeam;
            if (oldTeam && oldTeam !== "telekinesis") {
                Utils.server.runCommandSilent(`team join ${oldTeam} ${entityId}`);
            }
        });

        if (!entity.tags.contains("psyco_block_throw") && !(entity instanceof $Projectile)) {
            entity.persistentData.remove("telekinesisTargetedBy");
        }
    }
}


function handlePsycoShield(entity, entityData) {
    entityData.psyco_shield -= 1;
    if (entityData.psyco_shield <= 0) {
        entity.setNoGravity(false);
        entity.setGlowing(false);
        entityData.remove('psyco_shield');
        entity.addTag("psyco_shield_drop");
    }
}


function handlePsycoBlock(entity) {
    const requiredLevel = entity.persistentData.telekinesisRequiredLevel;
    const throwForce = entity.persistentData.telekinesisThrowForce;
    const damage = requiredLevel * throwForce;

    if (entity.tags.contains("psyco_block_throw") && !entity.nbt?.TileEntityData) {
        let areaEntities = entity.level.getEntitiesWithin(AABB.of(
            entity.x - 0.5, entity.y - 0.5, entity.z - 0.5,
            entity.x + 0.5, entity.y + 0.5, entity.z + 0.5
        ));

        areaEntities.forEach(ent => {
            const player = getEntityByUUID(entity.persistentData.telekinesisTargetedBy);
            if (ent && ent.isLiving() && player && player.uuid !== ent.uuid &&
                ent.id !== entity.id && ent.isAlive() &&
                !['minecraft:falling_block', 'minecraft:armor_stand'].includes(ent.type)) {
                const damageSource = player.damageSources().playerAttack(player);
                ent.attack(damageSource, damage);
                entity.mergeNbt({ Time: 600 });
            }
        });
    }
    Utils.server.scheduleInTicks(1, () => {
        entityDeadOrDying(entity);
    });
}


function entityDeadOrDying(entity) {
    if (entity.removed || entity.deadOrDying) {
        let nbtData = entity.getNbt();
        let { BlockState: { Name: id, Properties } } = nbtData;
        let block = entity.block;

        let itemEntity = entity.level.getEntitiesWithin(entity.boundingBox)
            .find(pEntity => pEntity.type == 'minecraft:item' &&
                pEntity.age <= 2 &&
                pEntity.getNbt().Item.id == id);

        if (nbtData.Time < 600 && itemEntity) {
            itemEntity.kill();
            if (block.hasTag(broke_falling)) block = block.up;

            entity.level.destroyBlock(block.pos, true, entity);
            block.set(id, {
                facing: Properties?.facing || null,
                half: Properties?.half || null,
                shape: Properties?.shape || null,
                face: Properties?.face || null,
                attachment: Properties?.attachment || null,
                axis: Properties?.axis || null,
            });

            if (block.entityData) {
                let blockItems = block.hasTag("telekinesis:storage")
                    ? entity.getNbt().TileEntityData.attachments[0]["items"]
                    : entity.getNbt().TileEntityData.Items;

                block.mergeEntityData({ attachments: [{ items: blockItems }] });
            }
        }
    }
}


function getEntityByUUID(uuid) {
    return Utils.server.getEntities().find(ent => ent.uuid == uuid);
}


function distanceBetween(player, entity) {
    const dx = entity.x - player.x;
    const dy = entity.y - player.eyeY;
    const dz = entity.z - player.z;

    
    const distance = Math.sqrt(((dx) ** 2) + ((dy) ** 2) + ((dz) ** 2));

    return distance;
}


function isWaterCauldron(block) {
    return block.id == "minecraft:water_cauldron";
}


function isCampfireLitUnderCauldron(cauldronBlock) {
    const blockBelow = cauldronBlock.level.getBlock(cauldronBlock.x, cauldronBlock.y - 1, cauldronBlock.z);
    return blockBelow.id == "minecraft:soul_campfire" && blockBelow.properties?.lit == "true";
}


function checkItemsInCauldron(entity, cauldronBlock) {
    const pos = cauldronBlock.pos;

    
    const itemsInCauldron = entity.level.getEntitiesWithin(
        AABB.of(pos.x, pos.y, pos.z, pos.x + 1, pos.y + 1, pos.z + 1)
    ).filter(entity => entity.type == "minecraft:item");

    
    const itemIds = itemsInCauldron.map(item => item.item.id);

    
    const requiredItems = global.telekinesisConfig.formulaItems;

    
    const itemsCount = {};
    itemIds.forEach(id => {
        itemsCount[id] = (itemsCount[id] || 0) + 1;
    });

    const allItemsPresent = requiredItems.every(requiredItem => {
        return itemsCount[requiredItem] >= 1; 
    });

    
    const isAboveLitCampfire = isCampfireLitUnderCauldron(cauldronBlock);

    if (allItemsPresent && isAboveLitCampfire) {
        
        performCauldronAction(entity, cauldronBlock);

        
        requiredItems.forEach(requiredItem => {
            const itemEntity = itemsInCauldron.find(item => item.item.id == requiredItem);
            if (itemEntity) {
                itemEntity.kill();
            }
        });
    }
}



function performCauldronAction(entity, cauldronBlock) {
    const closestPlayer = findClosestPlayer(entity, 2.5)
    let explosion = entity.block.createExplosion()
    explosion.strength(1)
    explosion.explode()
    reduceCauldronWaterLevel(cauldronBlock, 1)
    const campfireBlock = cauldronBlock.down;
    campfireBlock.set(campfireBlock.id, { lit: "false" });
    if(closestPlayer){
        closestPlayer.potionEffects.add("nausea", 215, 2, true, false);
        closestPlayer.potionEffects.add("slowness", 150, 2, true, false);
        closestPlayer.potionEffects.add("blindness", 150, 2, true, false);
        global.playSoundLocal(closestPlayer, 'minecraft:entity.elder_guardian.curse', "players", 1, 0.1);
        closestPlayer.persistentData.telekinesisEnergyCooldown = 40
        if(superpowerUtil.hasSuperpower(closestPlayer, "telekinesis:telekinesis")){
            superpowerUtil.removeSuperpower(closestPlayer, "telekinesis:telekinesis")
        } else superpowerUtil.addSuperpower(closestPlayer, "telekinesis:telekinesis")
    }
}


function reduceCauldronWaterLevel(cauldronBlock, amount) {
    const currentLevel = parseInt(cauldronBlock.properties.level || "0", 10); 
    const newLevel = Math.max(0, currentLevel - amount); 

    if (currentLevel > 0) {
        if (newLevel > 0) {
            
            cauldronBlock.set("minecraft:water_cauldron", { level: `${newLevel}` });
        } else {
            
            cauldronBlock.set("minecraft:cauldron");
        }
    }
}


function findClosestPlayer(entity, maxDistance) {
    const allPlayers = entity.level.getPlayers(); 
    let closestPlayer = null;
    let closestDistance = maxDistance; 

    allPlayers.forEach(player => {
        const distance = distanceBetween(entity, player);
        if (distance < closestDistance) {
            closestDistance = distance;
            closestPlayer = player;
        }
    });

    return closestPlayer; 
}